#!/usr/bin/env bash

SCRIPT_DIR=`dirname "$0"`
SCRIPT_DIR=`cd $SCRIPT_DIR; pwd;`
PKGROOT=${SCRIPT_DIR%/tools}

CONF_FILE="etc/loglistener.conf"
CONF_BAK="etc/loglistener.conf.bak"
CONF_TMP="etc/loglistener.conf.tmp"

DATA_DIR="./data"

# error
function print_err()
{
    echo -e "\033[1;31m[Error] [${1}] ${2}\033[0m"
    print_log "[Error] [$1] $2"
}

# warn
function print_warn()
{
    echo -e "\033[1;33m[Warn] ${1}\033[0m"
    print_log "[Warn] $1"
}

# output ok
function print_info()
{
    echo -e "\033[1;32m[OK] ${1}\033[0m"
    print_log "[OK] $1"
}
# output result
function print_result()
{
    echo -e "\033[1;32m[RESULT] ${1}\033[0m"
    print_log "[RESULT] $1"
}

# output help
function print_tip()
{
    do_help
}

# output timestamp log
function print_log()
{
    time=$(date "+%Y-%m-%d %H:%M:%S")
    echo "[$time] $1" >> log/loglistener_install.log
}

# output text log
function print_text_log()
{
    echo "$1" >> log/loglistener_install.log
}

function get_json_value()
{
    local json=$1
    local key=$2

    if [[ -z "$3" ]]; then
      local num=1
    else
      local num=$3
    fi

    local value=$(echo "${json}" | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/'${key}'\042/){print $(i+1)}}}' | tr -d '"' | sed -n ${num}p)
    echo ${value}
}

# check installed version
function test_installed()
{
    if [ -f /etc/init.d/loglistenerd ]; then
        read -p "There is a historical version installed, and continuing the installation will \
overwrite the old version. do you want to continue?(yes/no): " chs

        if [[ "$chs" == "yes" || "$chs" == "y" ]]; then
            :
        elif [[ "$chs" == "no" || "$chs" == "n" ]]; then
            echo "install interrupt"
            exit 0
        else
            echo "invalid input"
            exit 1
        fi
    fi
}

function check_root()
{
    if [ `id -u` -eq 0 ];then
	    :
    else
	    echo "No permission! You must have root privileges"
	    exit 1
    fi
}

# os version
function get_os_version()
{
    CENTOS="CentOS"
    UBUNTU="Ubuntu"
    DEBIAN="Debian"
    TLINUX="tlinux"
    OPENSUSE="openSUSE"
    COREOS="CoreOS"
    FREEBSD="FreeBSD"
    OTHER="otheros"

    VOS=$OTHER

    issue=`uname -a | tr A-Z a-z`

    if [ `echo $issue | grep debian | wc -l` -ge 1 ]; then
        VOS=$DEBIAN
        VERSION=`cat /etc/debian_version`
    elif [ `echo $issue | grep ubuntu | wc -l` -ge 1 ]; then
        VOS=$UBUNTU
        VERSION=`cat /etc/os-release | grep "VERSION_ID"  | sed -ne 's/[^0-9]*\([0-9]*\.*[0-9]*\).*/\1/p'`
    elif [ `echo $issue | grep tlinux | wc -l` -ge 1 ]; then
        VOS=$TLINUX
        VERSION=`cat /etc/os-release | grep "VERSION_ID"  | sed -ne 's/[^0-9]*\([0-9]*\.*[0-9]*\).*/\1/p'`
    elif [ `echo $issue | grep centos | wc -l` -ge 1 ]; then
        VOS=$CENTOS
        VERSION=`cat /etc/centos-release | sed -ne 's/[^0-9]*\([0-9]*\.[0-9]*\).*/\1/p'`
    elif [ `echo $issue | grep suse | wc -l` -ge 1 ]; then
        VOS=$OPENSUSE
        VERSION=`cat /etc/os-release | grep "VERSION_ID"  | sed -ne 's/[^0-9]*\([0-9]*\.*[0-9]*\).*/\1/p'`
    elif [ `echo $issue | grep coreos | wc -l` -ge 1 ]; then
        VOS=$COREOS
        VERSION=`cat /etc/os-release | grep "VERSION_ID"  | sed -ne 's/[^0-9]*\([0-9.]*\).*/\1/p'`
    elif [ `echo $issue | grep freebsd | wc -l` -ge 1 ]; then
        VOS=$FREEBSD
        VERSION=`uname -r | sed -ne 's/\([0-9]*\.*[0-9]*\)-RELEASE/\1/p'`
    fi

    if [ `uname -m | grep "64" | wc -l` -ge 1 ]; then
        PLATFORM="x64"
    else
        PLATFORM="x32"
        print_err "NotSupportOS" "Loglistener Only Support 64-bit OS"
        exit 1
    fi

    OS_ANAME="$VOS $VERSION $PLATFORM"
}

# check os
function test_os()
{
    print_log "begin check OS environment ..."

    get_os_version

    SUPPORT_OS_LIST=(
        "$CENTOS 6.8 x64"
        "$CENTOS 6.9 x64"
        "$CENTOS 7.2 x64"
        "$CENTOS 7.3 x64"
        "$CENTOS 7.4 x64"
        "$CENTOS 7.5 x64"
        "$CENTOS 7.6 x64"
        "$COREOS 1745.5.0 x64"
        "$DEBIAN 9.0 x64"
        "$DEBIAN 8.2 x64"
        "$OPENSUSE 42.3 x64"
        "$UBUNTU 18.04 x64"
        "$UBUNTU 16.04 x64"
        "$UBUNTU 14.04 x64"
        "$TLINUX 2.2 x64"
    )
    if [ `echo "${SUPPORT_OS_LIST[@]}" | grep "$OS_ANAME" | wc -l` -ge 1 ]
    then
        print_info "check OS environment ${OS_ANAME} ok"
    fi
}

# check running
function check_running()
{
    loglis_num=`ps -ef 2>/dev/null | grep -E "loglistenerm|loglisteneru|loglistener -" | grep -vE "grep" | wc -l`
    if [ $loglis_num -ge 1 ]; then
		return 0
	else
		return 1
	fi
}

# init help
function do_init_help()
{
    echo "Usage:"
    echo "  $cmd init [OPTION]"
    echo "      "
    echo "Examples:"
    echo "  # Auto get IP address"
    echo "  $cmd init --secretid AKIDc9YlmrBcFk4C8sbmXQ8i65XXXXXXXXXX --secretkey LUSE4nPK1d4tX5SHyXv6tZXXXXXXXXXX --region ap-beijing"
    echo "  # Specify IP address"
    echo "  $cmd init --secretid AKIDc9YlmrBcFk4C8sbmXQ8i65XXXXXXXXXX --secretkey LUSE4nPK1d4tX5SHyXv6tZXXXXXXXXXX --region ap-beijing --ip 10.0.0.0"
    echo "Options:"
    echo "  -s, --secretid    part of the cloud API key, secretid is used to identify the API caller identity"
    echo "  -k, --secretkey   part of the cloud API key, secretkey is the key used to encrypt the signature string and"
    echo "                    the server-side validation signature string"
    echo "  -r, --region      available region of access point, you can use ap-beijing to specify the region Beijing,"
    echo "                    you can view all supported regions in https://cloud.tencent.com/document/product/614/18940"
    echo "  -i, --ip          optional, if you don't specify, loglistener will obtain an IP address automatically in the"
    echo "                    following order: hostname(etc/hosts), eth0, eth1..., bond0, bond1..., other network interface"
    echo "  -n, --network     optional, use which network to access CLS, supported internet and intra, default intra."
    echo "  -l, --label       optional, define loglistener by labels, support multiple labels,separated by commas(',')"
    echo "  -h, --help        show this help"
}

# help
function do_help()
{
    echo "Usage:"
    echo "  $cmd [OPTITION] COMMAND"
    echo "Options:"
    echo "  -h, --help          show this help"
    echo "  -v, --version       show the version of loglistener"
    echo "Subcommands:"
    echo "        init          initialize parameters and generate configuration file"
    echo "        install       install loglistener"
    echo "        uninstall     uninstall loglistener，delete all data and files"
}

function do_install()
{
    # check installed version
    test_installed

    # if installeded version is running, exit
    check_running
    if [ $? -eq 0 ];then
        echo "loglistener is running, please stop loglistener before you reinstall"
        exit 1
    fi

    # check os support
    test_os

    # check loglistener dependency
    test_dependencies

    # add /etc/init.d/loglistenerd
    sed -e "s#BASE_DIR=.*#BASE_DIR=${PKGROOT}#" tools/loglistenerd.temple > /etc/init.d/loglistenerd
    chmod 755 /etc/init.d/loglistenerd

    if [ $VOS = $TLINUX ] || [ $VOS = $CENTOS ] || [ $VOS = $OPENSUSE ]; then
        chkconfig --add loglistenerd
    fi
    if [ -d "$DATA_DIR" ];then
        rm -rf $DATA_DIR
    fi

    # data dir
    mkdir -p $DATA_DIR

    print_result "install loglistener success，use '$cmd init' to init loglistener's config"
}

# get init param
function get_init_param()
{
    GETOPT_O="s:k:r:i:n:l:h"
    GETOPT_L="secretid:,secretkey:,region:,ip:,network:,label:,help"
    ARGS=`getopt -a -o $GETOPT_O  -l $GETOPT_L -- "$@"`
    [ $? -ne 0 ] && do_init_help && exit 1
    eval set -- "${ARGS}"
    while true
    do
        case "$1" in
            -s|--secretid)
                secretid="$2"
                shift
                ;;
            -k|--secretkey)
                secretkey="$2"
                shift
                ;;
            -r|--region)
                region="$2"
                shift
                ;;
            -i|--ip)
                ip="$2"
                shift
                ;;
            -n|--network)
                network="$2"
                shift
                ;;
            -l|--label)
                label="$2"
                shift
                ;;
            -h|--help)
                do_init_help
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    shift
    done
}

# check param region
function test_params_region()
{
    # add region prefix
    apregions="beijing shanghai guangzhou chengdu nanjing hongkong shenzhen-fsi shanghai-fsi"
    naregions="toronto ashburn siliconvalley"

    conf_regions=$region

    for item in $apregions
    do
        if [ $conf_regions = $item ]
        then
            conf_regions="ap-"$region
            break
        fi
    done

    for item in $naregions
    do
        if [ $conf_regions = $item ]
        then
            conf_regions="na-"$region
            break
        fi
    done

    # check region
    regions="ap-beijing ap-shanghai ap-guangzhou ap-chengdu ap-nanjing ap-hongkong na-toronto na-ashburn na-siliconvalley ap-shenzhen-fsi ap-shanghai-fsi"
    bfind=0
    for item in $regions
    do
        if [ $conf_regions = $item ]
        then
            bfind=1
            break
        fi
    done


    if [ $bfind -eq 0 ]
    then
        print_err "InvalidParam" "Invalid Region $conf_regions"
        do_init_help
        exit 1
    else
        print_info "check parameter region $conf_regions ok"
    fi
}

# update config file
function update_config_file()
{
    conf_file="$1"
    get_os_version
    if [ "$VERSION" = "$FREEBSD" ]; then
        sed -i '' "s/proxy_host *=.*/proxy_host = ${proxy_host}/" $conf_file
        sed -i '' "s/secret_id *=.*/secret_id = ${secretid}/" $conf_file
        sed -i '' "s/secret_key *=.*/secret_key = ${secretkey}/" $conf_file
        sed -i '' "s/group_ip *=.*/group_ip = ${group_ip}/" $conf_file
        sed -i '' "s/group_label *=.*/group_label = ${group_label}/" $conf_file
        sed -i '' "s/instance_id *=.*/instance_id = ${instance_id}/" $conf_file
    else
        sed -i "s/proxy_host *=.*/proxy_host = ${proxy_host}/" $conf_file
        sed -i "s/secret_id *=.*/secret_id = ${secretid}/" $conf_file
        sed -i "s/secret_key *=.*/secret_key = ${secretkey}/" $conf_file
        sed -i "s/group_ip *=.*/group_ip = ${group_ip}/" $conf_file
        sed -i "s/group_label *=.*/group_label = ${group_label}/" $conf_file
        sed -i "s/instance_id *=.*/instance_id = ${instance_id}/" $conf_file
    fi
}

# create config file
function create_config_file(){
    cp ${CONF_FILE}.temple ${CONF_TMP}
    update_config_file "${CONF_TMP}"
    mv ${CONF_TMP} ${CONF_FILE}
    chmod 0600 ${CONF_FILE}
}

# check dependency
function test_dependencies(){
    print_log "begin check dependencies ..."

    LSTLIB=`ldd bin/loglistener | grep "not found" | cut -f 1 -d ' '`

    if [ -z $LSTLIB ]
    then
        print_info "check dependencies ok"
    else
        print_err "MissingDependencies" "Missing Dependencies:\n${LSTLIB}"
        exit 1
    fi
}

# check network
function check_net()
{
    print_log "begin check network connection ..."
    hostip=`bin/gethostip "$1"`
    if [ $? -ne 0 ];then
        print_err "NetConnectError" "$hostip"
        exit 1
    fi

    if [ "$hostip" = "$1" ];then
        echo "Connect to $hostip ..."
    else
        echo "Connect to $1 ($hostip) ..."
    fi

    res=`bin/check_net "$hostip"`
    errno=$?

    if [ $errno -eq 0 ];then
        print_info "check network connection ok"
    else
        print_err "NetConnectError" "$res"
        exit 1
    fi
}

# check secretid and secretkey
function test_params_key()
{
    print_log "begin check authentication ..."

    chk=`bin/check_key ${1} ${2} ${3}`
    print_log "check result is:\n$chk\n"

    if [ `echo "$chk" | grep "curl http response code:200" | wc -l` -ge 1 ]
    then
        print_info 'check authentication ok'
    else
        js=`echo "$chk" | grep '{"errorcode'`
        errcode=$(get_json_value "$js" "errorcode")
        errmsg=$(get_json_value "$js" "errormessage")
        print_err "${errcode}" "${errmsg},please check the parameters secretid and secretkey"
        exit 1
    fi
}

function test_label()
{
    reg="^([a-zA-Z0-9]+[a-zA-Z0-9_-]*[,]*)+$"
    if [ ! -z $label ]
    then
        num=`echo $label | egrep $reg -c`
        if [ $num -ne 1 ]
        then
            print_err "InvalidParam" "Invalid label $label"
            exit 1
        fi
        group_label=$label
    fi
}

# check ip and auto get ip
function test_or_autoget_ip()
{
    reg="^(([0-9]{1,2}|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]{1,2}|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"
    print_log "begin get group ip ..."
    if [ ! -z $ip ]
    then
        num=`echo $ip | egrep $reg -c`
        if [ $num -ne 1 ]
        then
            print_err "InvalidParam" "Invalid ip $ip"
            exit 1
        fi
        group_ip=$ip
    else
        ips=(`bin/getip`)
        if [ ${#ips[@]}  -eq 1 ]
        then
            group_ip=${ips[0]}
        else
            echo "get ip is:${ips[@]}"

            while :
            do
                read -p "please enter your ip:" group_ip
                num=`echo ${group_ip} | egrep $reg -c`
                if [ $num -eq 1 ]
                then
                    break
                else
                    echo "input group ip error!"
                fi
            done
        fi
    fi

}

function get_param_from_conf_file(){
    conf_file="${CONF_FILE}"
    val=`cat $conf_file | grep -E "^\s*$1" -m 1 |  sed -r "s/\s*$1\s*=\s*(\S*)/\1/g"`
    echo "$val"
}

# check difference bettwen new init param and config file
function check_params_config_diff()
{
    old_proxy_host=$(get_param_from_conf_file "proxy_host")
    old_secret_id=$(get_param_from_conf_file "secret_id")
    old_secret_key=$(get_param_from_conf_file "secret_key")
    old_group_ip=$(get_param_from_conf_file "group_ip")
    old_group_label=$(get_param_from_conf_file "group_label")
    if [ "$proxy_host" != "$old_proxy_host" -o "$secretid" != "$old_secret_id" -o "$secretkey" != "$old_secret_key" -o "$group_ip" != "$old_group_ip" -o "$group_label" != "$old_group_label" ]
    then
        return 0
    else
        return 1
    fi
}

# refresh config file
function refresh_config()
{
    if [ -e ${CONF_FILE} ]
    then
        # if config file exist, check difference bettwen new init param and config file
        check_params_config_diff
        if [ $? -eq 0 ];then
            # if param updated, confirm
            read -p "config parameters change, data will be cleaned, do you want to continue? (yes/no): " choice
            if [[ "$choice" == "no" || "$choice" == "n" ]];then
                exit 1
            elif [[ "$choice" == "yes" || "$choice" == "y" ]];then
                :
            else
                echo "invalid input"
                exit 1
            fi

            # remove data file
            rm -rf $DATA_DIR/*

            # update param
            update_config_file "${CONF_FILE}"
        else
            echo "config parameters don't change, do nothing"
            exit 0
        fi
    else
        # if config file does not exist, create
        create_config_file
    fi
}

# check init param
function test_init_param()
{
    if [ -z $secretid ];then
        print_err "InvalidParam" "Missing Parameter secretid"
        do_init_help
        exit 1
    fi

    if [ -z $secretkey ];then
        print_err "InvalidParam" "Missing Parameter secretkey"
        do_init_help
        exit 1
    fi

    if [ -z $region ];then
        print_err "InvalidParam" "Missing Parameter region"
        do_init_help
        exit 1
    fi

    if [ -n "$network" ] && [ "$network" != "intra" -a "$network" != "internet" ];then
        print_err "InvalidParam" "Invalid Parameter network"
        do_init_help
        exit 1
    fi

    # # check region param
    # test_params_region

    # delete region check
    conf_regions=${region}

    # check loglistener dependency
    test_dependencies

    # check network
    if [ -n "$network" ];then
        if [ "$network" == "internet" ];then
            proxy_host="${conf_regions}.cls.tencentcs.com"
        else
            proxy_host="${conf_regions}.cls.tencentyun.com"
        fi
    else
        proxy_host="${conf_regions}.cls.tencentyun.com"
    fi

    check_net "$proxy_host"

    # check key
    test_params_key $secretid $secretkey "$proxy_host"

    # get ip
    test_or_autoget_ip

    # test label
    test_label
}

# init and generate config file
function do_init()
{
    # check installed
    if [ ! -f /etc/init.d/loglistenerd ]; then
        echo "loglistener has not been installed"
        exit 1
    fi

    # check running
    check_running
    if [ $? -eq 0 ];then
        echo "loglistener is running, please stop loglistener before you init"
        exit 1
    fi

    # get init param
    get_init_param "${@:1}"

    print_text_log ""
    print_text_log ""
    print_text_log "------------------------------------------------"
    print_log "begin init loglistener ..."
    print_log "begin check parameters ..."

    # test param
    test_init_param
    print_log "begin generate instance id ..."
    do_gen_instanceId

    # generate config file
    refresh_config
    print_result "loglistener init success, group ip is $group_ip"
}

# generate instance id
function do_gen_instanceId()
{
    which "dmidecode" >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		md5string=`dmidecode -t system | grep -E 'Product Name|Version|Serial Number|UUID' | tr '[\n\t |]' '-'`
    else
        macs=(`bin/getmac`)
        if [ ${#macs[@]}  -eq 1 ]
        then
            md5string=${macs[0]}
        else
            md5string=${group_ip}
        fi
    fi
    myuuid=`./bin/getmd5 "$md5string"`
	if [ $? -eq 0 ]; then
		instance_id=loglistener-$myuuid
		print_result "instance id gen success, instance id is $instance_id"
	else
		print_result "instance id gen failed"
	fi
	
}

# update config file
function do_config()
{
    :
}

function do_uninstall()
{
    # check running
    check_running
    if [ $? -eq 0 ]; then
        echo "loglistener is running, please stop it first"
        exit 1
    fi

    # confirm to uninstall
    read -p "uninstall will delete all data and config files, do you want to continue?(yes/no): " chs

    if [[ "$chs" == "yes" || "$chs" == "y" ]]; then
        :
    elif [[ "$chs" == "no" || "$chs" == "n" ]]; then
    	echo "uninstall interrupt"
    	exit 1
    else
    	echo "invalid input"
    	exit 1
    fi

    # check uninstalled
    if [ ! -f /etc/init.d/loglistenerd ]; then
        echo "loglistener has been uninstalled"
        exit 0
    fi

    # uninstall
    get_os_version

    # chkconfig off
    if [ $VOS = $TLINUX ] || [ $VOS = $CENTOS ] || [ $VOS = $OPENSUSE ]; then
        if [ -f /etc/init.d/loglistenerd ]; then
        	chkconfig --del loglistenerd
        fi
    fi

    # remove loglistenerd
    rm -f /etc/init.d/loglistenerd

    # remove config file
    if [ -f ${CONF_FILE} ]; then
        rm -rf ${CONF_FILE}
    fi

    # remove data
    if [ -d "$DATA_DIR" ]; then
        rm -rf $DATA_DIR
    fi
    print_info "uninstall loglistener success"
}

# update
function do_update()
{
    :
}

function do_version()
{
    version=`bin/loglistener -v`
	echo "loglistener-$version"
}

# check root
check_root

# enter loglistener directory
cd $PKGROOT

# log dir
mkdir -p log

# get command
cmd="$0"

# check subcommand
# help
# install
# init
# config
# uninstall
# update
# version
case "$1" in
    -h|--help)
        do_help
        ;;
    install)
        do_install
        ;;
    init)
        do_init $@
        ;;
    config)
        do_config $@    # TODO
        ;;
    uninstall)
        do_uninstall
        ;;
    update)
        do_update       # TODO
        ;;
    -v|--version)
        do_version
        ;;
    *)
        do_help
        ;;
esac
